###########################################################################
# Top30 is Copyright (C) 2016-2017 Kyle Robbertze <krobbertze@gmail.com>
#
# Top30 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# Top30 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Top30.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
"""
Runs the Rundown creator
"""
import argparse
from datetime import date
from datetime import timedelta

from top30.handlers import UserInterface
from top30.top_30_creator import Top30Creator
from top30.settings import Settings

SETTINGS = Settings()

def main():
    """
    Main function. Runs the program
    """
    parser = argparse.ArgumentParser()
    parser.add_argument("chart", help="chart to create the rundowns from")
    parser.add_argument("-p", "--previous", help="the previous chart")
    parser.add_argument("-g", "--gui", help="run the graphical version of Top30",
                        action="store_true")
    args = parser.parse_args()
    if args.previous != None:
        previous = args.previous
    else:
        current_date = args.chart[-15:-5].split("-")
        current = date(int(current_date[0]), int(current_date[1]),
                       int(current_date[2]))
        previous_date = current - timedelta(days=7)
        previous = args.chart[:-len("YYYY-MM-DD.docx")] + \
                previous_date.isoformat() + ".docx"

    if args.gui:
        gui = UserInterface()
        gui.run(args.chart, previous)
    else:
        creator = Top30Creator(args.chart, previous)
        print("Creating 30 - 21 rundown...")
        creator.create_rundown(30, 21, "")
        print("Creating 20 - 11 rundown...")
        creator.create_rundown(20, 11, "")
        print("Creating 10 - 2 rundown...")
        creator.create_rundown(10, 2, "")
        print("Creating last week's 10 - 1 rundown...")
        creator.create_rundown(10, 1, "last-week")



###########################################################################
# Top30 is Copyright (C) 2016-2017 Kyle Robbertze <krobbertze@gmail.com>
#
# Top30 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# Top30 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Top30.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
"""
Manages the settings that are used while running
"""
import configparser
import os.path

import top30

class Settings:
    """
    Holds the settings for the top30 module
    """
    def __init__(self):
        self.config = configparser.ConfigParser()
        config_file = find_config_file()
        self.config.read(config_file)

    def song_length(self):
        """Returns the length of the song section length"""
        return int(self.config['Song']['Length'])

    def song_start_tag(self):
        """Retuns the tag that indicates the start of the song section"""
        return self.config['Song']['StartTag']

    def song_directory(self):
        """Returns the directory of the songs"""
        song_directory = self.config['Song']['Directory']
        if song_directory.startswith("~"):
            return os.path.expanduser(song_directory)
        return song_directory

    def voice_start_overlap(self):
        """Returns the voice start overlap time"""
        return int(self.config['Voice']['StartOverlap'])

    def voice_end_overlap(self):
        """Returns the voice end overlap time"""
        return int(self.config['Voice']['EndOverlap'])

    def voice_directory(self):
        """Returns the directory of the voice"""
        voice_directory = self.config['Voice']['Directory']
        if voice_directory.startswith("~"):
            return os.path.expanduser(voice_directory)
        return voice_directory

def find_config_file():
    """
    Locates the config file. It uses the last one it finds in order:
    /etc/top30.conf
    <module_path>/top30.conf
    ~/.top30.conf
    ./top30.conf
    """
    if os.path.isfile("./top30.conf"):
        return "./top30.conf"
    if os.path.isfile(os.path.join( \
                      top30.__file__[:len("__init__.py") + 1], "top30.conf")):
        return os.path.join(top30.__file__[:len("__init__.py") + 1], "top30.conf")
    home = os.path.expanduser("~/.top30.conf")
    if os.path.isfile(home):
        return home
    if os.path.isfile("/etc/top30.conf"):
        return "/etc/top30.conf"
    raise FileNotFoundError("Unable to find config file")



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'main_window.ui'
#
# Created by: PyQt5 UI code generator 5.7
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(767, 454)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.btn_create = QtWidgets.QPushButton(self.centralwidget)
        self.btn_create.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.btn_create.setObjectName("btn_create")
        self.gridLayout.addWidget(self.btn_create, 1, 1, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btn_add_clip = QtWidgets.QPushButton(self.centralwidget)
        self.btn_add_clip.setObjectName("btn_add_clip")
        self.horizontalLayout.addWidget(self.btn_add_clip)
        self.btn_delete_clip = QtWidgets.QPushButton(self.centralwidget)
        self.btn_delete_clip.setObjectName("btn_delete_clip")
        self.horizontalLayout.addWidget(self.btn_delete_clip)
        self.gridLayout.addLayout(self.horizontalLayout, 1, 0, 1, 1)
        self.clip_view = ClipListView(self.centralwidget)
        self.clip_view.setLineWidth(1)
        self.clip_view.setObjectName("clip_view")
        self.gridLayout.addWidget(self.clip_view, 0, 0, 1, 1)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.btn_move_up = QtWidgets.QPushButton(self.centralwidget)
        self.btn_move_up.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.btn_move_up.setObjectName("btn_move_up")
        self.verticalLayout_2.addWidget(self.btn_move_up)
        self.btn_move_down = QtWidgets.QPushButton(self.centralwidget)
        self.btn_move_down.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.btn_move_down.setObjectName("btn_move_down")
        self.verticalLayout_2.addWidget(self.btn_move_down)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setFieldGrowthPolicy(QtWidgets.QFormLayout.AllNonFixedFieldsGrow)
        self.formLayout.setObjectName("formLayout")
        self.lbl_song_length = QtWidgets.QLabel(self.centralwidget)
        self.lbl_song_length.setObjectName("lbl_song_length")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.lbl_song_length)
        self.txt_song_length = QtWidgets.QLineEdit(self.centralwidget)
        self.txt_song_length.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.txt_song_length.setObjectName("txt_song_length")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.txt_song_length)
        self.lbl_voice_start = QtWidgets.QLabel(self.centralwidget)
        self.lbl_voice_start.setObjectName("lbl_voice_start")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.lbl_voice_start)
        self.txt_voice_start = QtWidgets.QLineEdit(self.centralwidget)
        self.txt_voice_start.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.txt_voice_start.setObjectName("txt_voice_start")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.txt_voice_start)
        self.lbl_voice_end = QtWidgets.QLabel(self.centralwidget)
        self.lbl_voice_end.setObjectName("lbl_voice_end")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.lbl_voice_end)
        self.txt_voice_end = QtWidgets.QLineEdit(self.centralwidget)
        self.txt_voice_end.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.txt_voice_end.setObjectName("txt_voice_end")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.txt_voice_end)
        self.verticalLayout_2.addLayout(self.formLayout)
        self.gridLayout.addLayout(self.verticalLayout_2, 0, 1, 1, 1)
        self.gridLayout.setColumnStretch(0, 1)
        self.gridLayout.setRowStretch(1, 1)
        self.gridLayout_2.addLayout(self.gridLayout, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 767, 21))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuEdit = QtWidgets.QMenu(self.menubar)
        self.menuEdit.setObjectName("menuEdit")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.act_new = QtWidgets.QAction(MainWindow)
        self.act_new.setObjectName("act_new")
        self.act_create_clip = QtWidgets.QAction(MainWindow)
        self.act_create_clip.setObjectName("act_create_clip")
        self.act_exit = QtWidgets.QAction(MainWindow)
        self.act_exit.setObjectName("act_exit")
        self.act_add_clip = QtWidgets.QAction(MainWindow)
        self.act_add_clip.setObjectName("act_add_clip")
        self.act_delete_clip = QtWidgets.QAction(MainWindow)
        self.act_delete_clip.setObjectName("act_delete_clip")
        self.act_help = QtWidgets.QAction(MainWindow)
        self.act_help.setObjectName("act_help")
        self.act_about = QtWidgets.QAction(MainWindow)
        self.act_about.setObjectName("act_about")
        self.menuFile.addAction(self.act_new)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.act_create_clip)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.act_exit)
        self.menuEdit.addAction(self.act_add_clip)
        self.menuEdit.addAction(self.act_delete_clip)
        self.menuHelp.addAction(self.act_help)
        self.menuHelp.addSeparator()
        self.menuHelp.addAction(self.act_about)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuEdit.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Top 30 Creator"))
        self.btn_create.setText(_translate("MainWindow", "Create"))
        self.btn_add_clip.setText(_translate("MainWindow", "Add Clip"))
        self.btn_delete_clip.setText(_translate("MainWindow", "Delete Clip"))
        self.btn_move_up.setText(_translate("MainWindow", "Move Up"))
        self.btn_move_down.setText(_translate("MainWindow", "Move Down"))
        self.lbl_song_length.setText(_translate("MainWindow", "Song Length:"))
        self.lbl_voice_start.setText(_translate("MainWindow", "Voice Start Overlap:"))
        self.lbl_voice_end.setText(_translate("MainWindow", "Voice End Overlap:"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.menuEdit.setTitle(_translate("MainWindow", "Edit"))
        self.menuHelp.setTitle(_translate("MainWindow", "Help"))
        self.act_new.setText(_translate("MainWindow", "New"))
        self.act_create_clip.setText(_translate("MainWindow", "Create"))
        self.act_exit.setText(_translate("MainWindow", "Exit"))
        self.act_add_clip.setText(_translate("MainWindow", "Add Clip..."))
        self.act_delete_clip.setText(_translate("MainWindow", "Delete Clip.."))
        self.act_help.setText(_translate("MainWindow", "Top 30 Creator Help"))
        self.act_about.setText(_translate("MainWindow", "About"))

from top30.clip_list import ClipListView



###########################################################################
# Top30 is Copyright (C) 2016-2017 Kyle Robbertze <krobbertze@gmail.com>
#
# Top30 is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# Top30 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Top30.  If not, see <http://www.gnu.org/licenses/>.
###########################################################################
"""
Creates rundowns from a chart
"""
from mutagen.oggvorbis import OggVorbis
from pydub import AudioSegment

import top30
from top30.chart import Chart

class Top30Creator:
    """
    Class for creating rundowns from a start to stop
    """
    def __init__(self, chart_file, last_week):
        self.voice_begin_overlap = top30.SETTINGS.voice_start_overlap()
        self.voice_end_overlap = top30.SETTINGS.voice_end_overlap()
        self.song_length = top30.SETTINGS.song_length()
        self.chart = Chart(chart_file)
        self.last_week = Chart(last_week)

    def create_rundown(self, start, end, is_last_week):
        """
        Creates a rundown from start to end (both inclusive). If there is a
        directory that is added to the file paths and the intro/outro names.
        """
        voice_dir = top30.SETTINGS.voice_directory()
        prefix = ""
        current_chart = self.chart
        if is_last_week:
            prefix = "last-week_" # Intro/outro handling
            current_chart = self.last_week

        intro = "%s/%s%02d-%02d_intro.ogg" % (voice_dir, prefix, start, end)
        rundown = get_start(intro)
        song_file = current_chart.get(start, 'path')
        rundown = self.add_song(song_file, rundown)

        for i in range(start - 1, end - 1, -1):
            voice_file = voice_dir + "/" + str(i) + ".ogg"
            rundown = self.add_voice(voice_file, rundown)
            song_file = current_chart.get(i, 'path')
            rundown = self.add_song(song_file, rundown)

        outro = "%s/%s%02d-%02d_outro.ogg" % (voice_dir, prefix, start, end)
        rundown = add_end(outro, rundown)
        rundown_name = "rundown-%s%02d-%02d" % (prefix, start, end)
        export(rundown_name, "mp3", rundown)

    def add_voice(self, voice_file, rundown):
        """
        Adds a voice segment to the rundown
        """
        voice = AudioSegment.from_ogg(voice_file)
        rundown = rundown.overlay(voice[:self.voice_begin_overlap],
                                  position=-self.voice_begin_overlap)
        return rundown.append(voice[self.voice_begin_overlap:],
                              crossfade=0.5*1000)

    def add_song(self, song_file, rundown):
        """
        Adds a song segment to the rundown
        """
        start_time = get_start_time(song_file)

        song = AudioSegment.from_ogg(song_file)
        song = song[start_time:start_time + self.song_length]
        song = song.overlay(rundown[-self.voice_end_overlap:])
        return rundown[:-self.voice_end_overlap].append(song, crossfade=0)

    def get_song_length(self):
        """
        Returns the length of the song segment set in the config file
        """
        return self.song_length

    def get_voice_begin_overlap(self):
        """
        Returns the length of the voice overlap set in the config file
        """
        return self.voice_begin_overlap

    def get_voice_end_overlap(self):
        """
        Returns the length of the voice overlap set in the config file
        """
        return self.voice_end_overlap

    def set_song_length(self, song_length):
        """
        Sets the length of the song segment set in the config file
        """
        self.song_length = song_length

    def set_voice_begin_overlap(self, voice_begin_overlap):
        """
        Sets the length of the voice overlap set in the config file
        """
        self.voice_begin_overlap = voice_begin_overlap

    def set_voice_end_overlap(self, voice_end_overlap):
        """
        Sets the length of the voice overlap set in the config file
        """
        self.voice_end_overlap = voice_end_overlap

def get_start_time(filename):
    """
    Returns the start time of song segment in miliseconds. This is read
    from the file's metadata
    """
    song_meta = OggVorbis(filename).tags
    tag = top30.SETTINGS.song_start_tag().lower()
    try:
        time_code = song_meta[tag][0]
    except KeyError:
        time_code = song_meta['comment'][0]
    song_length = float(time_code.split(':')[0]) * 60 + \
                  float(time_code.split(':')[1])
    song_length *= 1000
    return song_length


def get_start(filename):
    """
    Returns the intro voice segment
    """
    return AudioSegment.from_ogg(filename)

def add_end(filename, rundown):
    """
    Adds the outro voice segment
    """
    outro = AudioSegment.from_ogg(filename)
    return rundown.append(outro, crossfade=0)

def export(filename, file_type, rundown):
    """
    Exports the rundown as an audio file
    """
    if not filename.endswith("." + file_type):
        filename = filename + "." + file_type
    rundown.export(filename, format=file_type)
